/*
 *   logger - un loggeur
 *  Copyright (C) 2024  Valentin Moguérou
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdarg.h>
#include <string.h>

#include "logger.h"
Logger logger;

void create_logger(char* logpath, LogLevel level) {
    logger = malloc(sizeof(*logger));

    logger->t0 = clock();
    logger->level = level;
    
    if (logpath[0] != '\0')
    {
        logger->logfile = fopen(logpath, "w");
    }
    else
        logger->logfile = NULL;
}

void destroy_logger() {
    if (logger->logfile != NULL)
        fclose(logger->logfile);
    free(logger);
}

void logprint(LogLevel level, char* str, va_list args)
{
    clock_t t = clock();

    char label[6];
    char ansi_code[12];

    switch (level)
    {
        case (LOG_PANIC):
            strcpy(label, "PANIC");
            strcpy(ansi_code, "\x1b[31;49m");
            break;
        case (LOG_FATAL):
            strcpy(label, "FATAL");
            strcpy(ansi_code, "\x1b[31;49m");
            break;
        case (LOG_ERROR):
            strcpy(label, "ERROR");
            strcpy(ansi_code, "\x1b[31;49m");
            break;
        case (LOG_WARN):
            strcpy(label, "WARN");
            strcpy(ansi_code, "\x1b[93;49m");
            break;
        case (LOG_INFO):
            strcpy(label, "INFO");
            strcpy(ansi_code, "\x1b[33;49m");
            break;
        case (LOG_DEBUG):
            strcpy(label, "DEBUG");
            strcpy(ansi_code, "\x1b[96;49m");
            break;
        case (LOG_TRACE):
            strcpy(label, "TRACE");
            strcpy(ansi_code, "\x1b[36;49m");
            break;
    }

    FILE* stream = (level >= LOG_WARN) ? stderr : stdout;

    if (level < logger->level)
        return;

    fprintf(stream, "%s[%s][%f]: ", ansi_code, label, (double)t / CLOCKS_PER_SEC);
    vfprintf(stream, str, args);
    fprintf(stream, "\n");

    if (logger->logfile != NULL)
    {
        fprintf(logger->logfile, "[%s][%f]: ", label, (double)t / CLOCKS_PER_SEC);
        vfprintf(logger->logfile, str, args);
        fprintf(logger->logfile, "\n");
    }
}


void panic(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_PANIC, str, args);
    va_end(args);
}

void fatal(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_FATAL, str, args);
    va_end(args);
}

void error(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_ERROR, str, args);
    va_end(args);
}

void warn(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_WARN, str, args);
    va_end(args);
}

void info(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_INFO, str, args);
    va_end(args);
}

void debug(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_DEBUG, str, args);
    va_end(args);
}

void trace(char *str, ...)
{
    va_list args;
    va_start(args, str);
    logprint(LOG_TRACE, str, args);
    va_end(args);
}
